﻿using Model;

namespace Stub;

public class StubDataManager : IDataManager
{
    private readonly static List<Book> books = new()
    {
        new Book(1, "Les Trois Mousquetaires", "Alexandre Dumas", "979-8415441792"),
        new Book(2, "Vingt Ans Après", "Alexandre Dumas", "979-8805372019"),
        new Book(3, "Le Vicomte de Bragelonne - Livre Premier", "Alexandre Dumas", "979-8376127575"),
        new Book(4, "Le Vicomte de Bragelonne - Livre Second", "Alexandre Dumas", "979-8376430019"),
        new Book(5, "Vingt mille lieues sous les Mers", "Jules Verne", "979-8489937313"),
        new Book(6, "L'île mystérieuse", "Jules Verne", "979-8492533861"),
        new Book(7, "Les enfants du capitaine Grant", "Jules Verne", "979-8757717418"),
        new Book(8, "Les Misérables", "Victor Hugo", "979-8850172916"),
        new Book(9, "Notre-Dame de Paris", "Victor Hugo", "979-8863757285"),
        new Book(10, "Quatrevingt-treize", "Victor Hugo", "979-8863954202"),
        new Book(11, "Han d'Islande", "Victor Hugo", "979-8864251553"),
    };

    private readonly static List<Person> persons = new()
    {
        new Person(1, "Chick", "Corea"),
        new Person(2, "Stanley", "Clarke"),
        new Person(3, "Lenny", "White"),
        new Person(4, "Frank", "Gambale"),
        new Person(5, "Jean-Luc", "Ponty"),
        new Person(6, "Al", "Di Meola"),
        new Person(7, "Steve", "Gadd"),
    };

    private long bookId = 12;

    private long personId = 8;

    static StubDataManager()
    {
        books[0].Borrower = persons[0];
        persons[0].BorrowBook(books[0]);

        books[1].Borrower = persons[0];
        persons[0].BorrowBook(books[1]);

        books[2].Borrower = persons[0];
        persons[0].BorrowBook(books[2]);

        books[3].Borrower = persons[0];
        persons[0].BorrowBook(books[3]);

        books[4].Borrower = persons[1];
        persons[1].BorrowBook(books[4]);

        books[5].Borrower = persons[1];
        persons[1].BorrowBook(books[5]);

        books[6].Borrower = persons[1];
        persons[1].BorrowBook(books[6]);

        books[8].Borrower = persons[2];
        persons[2].BorrowBook(books[8]);

        books[10].Borrower = persons[3];
        persons[3].BorrowBook(books[10]);
    }

    public Task<bool> BorrowBook(Book book, Person person)
    {
        var foundBook = GetBookById(book.Id).Result;
        var foundPerson = GetPersonById(person.Id).Result;

        if(foundBook == null || foundPerson == null)
            return Task.FromResult(false);

        if(foundBook.Borrower != null)
        {
            return Task.FromResult(false);
        }
        foundBook.Borrower = foundPerson;
        foundPerson.BorrowBook(foundBook);
        return Task.FromResult(true);
    }

    public Task<Book> CreateBook(string title, string author, string isbn)
    {
        Book book = new Book(bookId, title, author, isbn);
        books.Add(book);
        bookId++;
        return Task.FromResult(book);
    }

    public Task<Person> CreatePerson(string firstName, string lastName)
    {
        Person person = new Person(personId, firstName, lastName);
        persons.Add(person);
        personId++;
        return Task.FromResult(person);
    }

    public Task<bool> DeleteBook(long id)
    {
        Book? book = books.SingleOrDefault(b => b.Id == id);
        if(book == null)
        {
            return Task.FromResult(false);
        }
        book.Borrower?.ReturnBook(book);
        books.Remove(book);
        return Task.FromResult(true);
    }

    public Task<bool> DeletePerson(long id)
    {
        Person? person = persons.SingleOrDefault(p => p.Id == id);
        if(person == null)
        {
            return Task.FromResult(false);
        }
        foreach(var book in person.Books)
        {
            book.Borrower = null;
        }
        persons.Remove(person);
        return Task.FromResult(true);
    }

    public Task<Book?> GetBookById(long id)
    {
        return Task.FromResult(books.SingleOrDefault(b => b.Id == id));
    }

    public Task<IEnumerable<Book>> GetBooks(int index, int count)
    {
        var foundBooks = books.Skip(index*count).Take(count);
        return Task.FromResult(foundBooks);
    }

    public Task<IEnumerable<Book>> GetBooksBorrowedBy(Person person, int index, int count)
    {
        var foundBooks = books.Where(b => person.Equals(b.Borrower))
                              .Skip(index*count).Take(count);
        return Task.FromResult(foundBooks);
    }

    public Task<IEnumerable<Book>> GetBooksByAuthor(string author, int index, int count)
    {
        var foundBooks = books.Where(b => b.Author.Contains(author, StringComparison.InvariantCultureIgnoreCase))
                              .Skip(index*count).Take(count);
        return Task.FromResult(foundBooks);
    }

    public Task<IEnumerable<Book>> GetBooksByIsbn(string isbn, int index, int count)
    {
        var foundBooks = books.Where(b => b.Isbn.Equals(isbn))
                              .Skip(index*count).Take(count);
        return Task.FromResult(foundBooks);
    }

    public Task<IEnumerable<Book>> GetBooksByTitle(string title, int index, int count)
    {
        var foundBooks = books.Where(b => b.Title.Contains(title, StringComparison.InvariantCultureIgnoreCase))
                              .Skip(index*count).Take(count);
        return Task.FromResult(foundBooks);
    }

    public Task<Person?> GetPersonById(long id)
    {
        return Task.FromResult(persons.SingleOrDefault(p => p.Id == id));
    }

    public Task<IEnumerable<Person>> GetPersons(int index, int count)
    {
        var foundPersons = persons.Skip(index*count).Take(count);
        return Task.FromResult(foundPersons);
    }

    public Task<IEnumerable<Person>> GetPersonsByName(string name, int index, int count)
    {
        var foundPersons = persons.Where(b => b.FirstName.Contains(name, StringComparison.InvariantCultureIgnoreCase)
                                            || b.LastName.Contains(name, StringComparison.InvariantCultureIgnoreCase))
                              .Skip(index*count).Take(count);
        return Task.FromResult(foundPersons);
    }

    public Task<bool> ReturnBook(Book book, Person person)
    {
        var foundBook = GetBookById(book.Id).Result;
        var foundPerson = GetPersonById(person.Id).Result;

        if(foundBook == null || foundPerson == null)
            return Task.FromResult(false);

        if(!foundPerson.Books.Contains(foundBook) || !foundPerson.Equals(foundBook.Borrower))
            return Task.FromResult(false);

        foundPerson.ReturnBook(foundBook);
        foundBook.Borrower = null;
        return Task.FromResult(true);
    }

    public Task<Book?> UpdateBook(long id, Book book)
    {
        var foundBook = GetBookById(id).Result;

        if(foundBook == null)
            return Task.FromResult((Book?)null);

        foundBook.Author = book.Author;
        foundBook.Isbn = book.Isbn;
        foundBook.Title = book.Title;
        return Task.FromResult((Book?)foundBook);
    }

    public Task<Person?> UpdatePerson(long id, Person person)
    {
        var foundPerson = GetPersonById(id).Result;

        if(foundPerson == null)
            return Task.FromResult((Person?)null);

        foundPerson.FirstName = person.FirstName;
        foundPerson.LastName = person.LastName;
        return Task.FromResult((Person?)foundPerson);
    }
}
